/**********************************************************************\

File:		msg menus.c

Purpose:	This module handles menu selections, including selection
			of dimmed menu items (hehe).


MSG Demo -- graphic effects demonstration program
Copyright (C) 1992-4 Mark Pilgrim & Dave Blumenthal

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program in a file named "GNU General Public License".
If not, write to the Free Software Foundation, 675 Mass Ave,
Cambridge, MA 02139, USA.

\**********************************************************************/

#include "msg menus.h"
#include "msg main.h"
#include "msg graphics.h"
#include "msg about.h"
#include "msg help.h"
#include "msg prefs.h"
#include "msg sounds.h"
#include "msg environment.h"
#include "demo crash.h"
#include "demo.h"
#include "program globals.h"

extern	long		menuDisable : 0x0b54;

Boolean			gMenuEnabled;
MenuHandle		gAppleMenu;
MenuHandle		gFileMenu;
MenuHandle		gEditMenu;
MenuHandle		gWipesMenu;
MenuHandle		gAdditionsMenu;
MenuHandle		gScrollMenu;
MenuHandle		gFluffMenu;
MenuHandle		gHelpMenu;
MenuHandle		gCrashMenu;

void AdjustMenus(void)
{
	WindowPeek	theWindow;
	int			kind;
	
	theWindow = (WindowPeek)FrontWindow();
	kind = theWindow ? theWindow->windowKind : 0;
	
	if(kind < 0)
		EnableItem(gEditMenu, 0);
	else
		DisableItem(gEditMenu, 0);
	
	if(theWindow)
		EnableItem(gFileMenu, closeItem);
	else
		DisableItem(gFileMenu, closeItem);
	
	if(gTheWindow[kMainWindow])
		DisableItem(gFileMenu, openItem);
	else
		EnableItem(gFileMenu, openItem);
	
	if (gDisableQuit)
	{
		DisableItem(gFileMenu, quitItem);
		DisableItem(gFileMenu, disableQuitItem);
		EnableItem(gFileMenu, enableQuitItem);
	}
	else
	{
		EnableItem(gFileMenu, quitItem);
		EnableItem(gFileMenu, disableQuitItem);
		DisableItem(gFileMenu, enableQuitItem);
	}
	if (gLastWipe==-1)
		DisableItem(gFluffMenu, repeatItem);
	else
		EnableItem(gFluffMenu, repeatItem);
	
	if (gWipeStatus==kFullScreen)
	{
		DisableItem(gWipesMenu, caste1Item);
		DisableItem(gWipesMenu, rippleItem);
		DisableItem(gWipesMenu, dissolveItem);
		DisableItem(gWipesMenu, caste2Item);
	}
	else
	{
		EnableItem(gWipesMenu, caste1Item);
		EnableItem(gWipesMenu, rippleItem);
		EnableItem(gWipesMenu, dissolveItem);
		EnableItem(gWipesMenu, caste2Item);
	}
	
	if (gIsReversed)
	{
		DisableItem(gWipesMenu, spiralItem);
		DisableItem(gAdditionsMenu, serendipityItem);
		DisableItem(gAdditionsMenu, bulgeItem);
		DisableItem(gScrollMenu, scrollInItem);
		DisableItem(gScrollMenu, splitScrollItem);
	}
	else
	{
		EnableItem(gWipesMenu, spiralItem);
		EnableItem(gAdditionsMenu, serendipityItem);
		EnableItem(gAdditionsMenu, bulgeItem);
		EnableItem(gScrollMenu, scrollInItem);
		EnableItem(gScrollMenu, splitScrollItem);
	}
	
	CheckItem(gFluffMenu, effectsOnlyItem, (gWipeStatus==kEffectsOnly));
	CheckItem(gFluffMenu, effectsPlusFadesItem, (gWipeStatus==kFadesPlusEffects));
	CheckItem(gFluffMenu, reverseEffectsPlusFadesItem, (gWipeStatus==kFadesPlusReverseEffects));
	CheckItem(gFluffMenu, fullScreenFadesItem, (gWipeStatus==kFullScreen));
	CheckItem(gFluffMenu, soundToggle, gSoundToggle&&gSoundAvailable);
	if (gSoundAvailable)
		EnableItem(gFluffMenu, soundToggle);
	else
		DisableItem(gFluffMenu, soundToggle);
}

void HandleMenu(long mSelect)
{
	int			menuID = HiWord(mSelect);
	int			menuItem = LoWord(mSelect);
	
	if (menuID==0)
	{
		menuID=HiWord(menuDisable);
		menuItem=LoWord(menuDisable);
		gMenuEnabled=FALSE;
	}
	else gMenuEnabled=TRUE;
	menuDisable=0L;

	switch (menuID)
	{
		case appleMenu:
			HandleAppleMenu(menuItem);
			break;
		case fileMenu:
			HandleFileMenu(menuItem);
			break;	
		case editMenu:
			HandleEditMenu(menuItem);
			break;
		case wipesMenu:
			HandleWipesMenu(menuItem);
			break;
		case additionsMenu:
			HandleAdditionsMenu(menuItem);
			break;
		case scrollMenu:
			HandleScrollMenu(menuItem);
			break;
		case helpMenu:
			HandleHelpMenu(menuItem);
			break;
		case fluffMenu:
			HandleFluffMenu(menuItem);
			break;
		case crashMenu:
			HandleCrashMenu(menuItem);
			break;
	}
}

void HandleAppleMenu(int menuItem)
{
	GrafPtr		savePort;
	Str255		name;
	
	if(menuItem == 1)
		ShowInformation();
	if (menuItem == 2)
		ShowSplashScreen();
	else if(menuItem > 4)
	{
		GetPort(&savePort);
		GetItem(gAppleMenu, menuItem, name);
		OpenDeskAcc(name);
		SetPort(savePort);
	}
}

void HandleFileMenu(int menuItem)
{
	WindowPtr			theWindow;
	int					i;
	Boolean				gotone;
	
	switch (menuItem)
	{
		case openItem:
			if (gMenuEnabled)
				OpenTheWindow(kMainWindow);
			else
				DoSound(sound_fluff, TRUE);
			break;
		case closeItem:
			if (gMenuEnabled)
			{
				theWindow=FrontWindow();
				gotone=FALSE;
				for (i=0; (i<NUM_WINDOWS) && (!gotone); i++)
					gotone=(theWindow==gTheWindow[i]);
					
				if (gotone)
					CloseTheWindow(i-1);
				else
					DisposeWindow(theWindow);
				
				AdjustMenus();
			}
			else DoSound(sound_fluff, TRUE);
			break;
		case disableQuitItem:
			if (gMenuEnabled)
			{
				gDisableQuit=TRUE;
				AdjustMenus();
			}
			else DoSound(sound_fluff, TRUE);
			break;
		case enableQuitItem:
			if (gMenuEnabled)
			{
				gDisableQuit=FALSE;
				AdjustMenus();
			}
			else DoSound(sound_fluff, TRUE);
			break;
		case quitItem:
			gDone = TRUE;
			break;
	}
}

void HandleEditMenu(int menuItem)
{
	if ((menuItem>0) && (menuItem!=2))
	{
		if (gMenuEnabled)
			SystemEdit(menuItem - 1);
		else DoSound(sound_fluff, TRUE);
	}
}

void HandleWipesMenu(int menuItem)
{
	if ((menuItem>0) && (menuItem<=13))
	{
		if (gMenuEnabled)
		{
			if (gWipeStatus==kFullScreen)
				DoFullScreenFade(menuItem, TRUE);
			else
			{
				OpenTheWindow(kMainWindow);
				gWhichWipe=menuItem;
				OpenTheWindow(kMainWindow);
			}
		}
		else DoSound(sound_fluff, TRUE);
	}
}

void HandleAdditionsMenu(int menuItem)
{
	if ((menuItem>0) && (menuItem<=11))
	{
		if (gMenuEnabled)
		{
			if (gWipeStatus==kFullScreen)
				DoFullScreenFade(menuItem+13, TRUE);
			else
			{
				OpenTheWindow(kMainWindow);
				gWhichWipe=menuItem+13;
				OpenTheWindow(kMainWindow);
			}
		}
		else DoSound(sound_fluff, TRUE);
	}
}

void HandleScrollMenu(int menuItem)
{
	if ((menuItem>0) && (menuItem<=8))
	{
		if (gMenuEnabled)
		{
			if (gWipeStatus==kFullScreen)
				DoFullScreenFade(menuItem+24, TRUE);
			else
			{
				OpenTheWindow(kMainWindow);
				gWhichWipe=menuItem+24;
				OpenTheWindow(kMainWindow);
			}
		}
		else DoSound(sound_fluff, TRUE);
	}
}

void HandleHelpMenu(int menuItem)
{
	if ((menuItem>0) && (menuItem<=gNumHelp))
	{
		gWhichHelp=menuItem;
		UpdateHelpWindow();
	}
}

void HandleFluffMenu(int menuItem)
{
	switch (menuItem)
	{
		case repeatItem:
			if (gMenuEnabled)
			{
				if (gWipeStatus==kFullScreen)
					DoFullScreenFade(gLastWipe, TRUE);
				else
				{
					OpenTheWindow(kMainWindow);
					gWhichWipe=gLastWipe;
					OpenTheWindow(kMainWindow);
				}
			}
			else DoSound(sound_fluff, TRUE);
			break;
		case soundToggle:
			if (gMenuEnabled)
			{
				gSoundToggle=!gSoundToggle;
				SaveThePrefs();
				DoSound(sound_on, TRUE);
			}
			break;
		case effectsOnlyItem:
			gWipeStatus=kEffectsOnly;
			SaveThePrefs();
			break;
		case effectsPlusFadesItem:
			gWipeStatus=kFadesPlusEffects;
			SaveThePrefs();
			break;
		case reverseEffectsPlusFadesItem:
			gWipeStatus=kFadesPlusReverseEffects;
			SaveThePrefs();
			break;
		case fullScreenFadesItem:
			gWipeStatus=kFullScreen;
			SaveThePrefs();
			break;
		case reverseToggle:
			gIsReversed=!gIsReversed;
			SaveThePrefs();
			ReverseAllWipes();
			break;
	}
}

void HandleCrashMenu(int menuItem)
{
	if (menuItem>0)
		CrashAndBurn(menuItem);
}

void ReverseAllWipes(void)
{
	int				whichMenu, whichItem, i;
	MenuHandle		thisMenu;
	Str255			thisName;
	char			thisChar;
	
	for (whichMenu=0; whichMenu<3; whichMenu++)
	{
		switch (whichMenu)
		{
			case 0:	thisMenu=gWipesMenu;		break;
			case 1:	thisMenu=gAdditionsMenu;	break;
			case 2:	thisMenu=gScrollMenu;		break;
		}
		
		for (whichItem=CountMItems(thisMenu); whichItem>0; whichItem--)
		{
			GetItem(thisMenu, whichItem, thisName);
			for (i=thisName[0]; i>thisName[0]/2; i--)
			{
				thisChar=thisName[i];
				thisName[i]=thisName[thisName[0]-i+1];
				thisName[thisName[0]-i+1]=thisChar;
			}
			SetItem(thisMenu, whichItem, thisName);
		}
	}
}
